(* ------------------------- INPUT.INC ------------------------------- *)
(*  Include-File fuer Eingabebefehle auf der JOYCE unter Turbo Pascal. *)
(*  Es sind enthalten:                                                 *)
(*                                                                     *)
(*     procedure WriteKeyMessage -> Schreibt eine Tastendruck-Meldung  *)
(*     procedure WaitForKey      -> Wartet auf einen Tastendruck       *)
(*     function  InputChar       -> Eingabe eines einzelnen Zeichens   *)
(*     procedure InputLine       -> Eingabe einer Zeile                *)
(*     procedure InputFileName   -> Eingabe eines Dateinamens (8.3)    *)
(* ------------------------------------------------------------------- *)

(*$V-*)

type WorkString = string(.255.);
type FileNameString = string(.12.);


(* Writes a 'Press a key' message to the screen. *)
procedure WriteKeyMessage;
begin
  Write('<Press a key>');
end;


(* Waits for a key to be pressed. *)
procedure WaitForKey;
var ch: Char;
begin
  repeat until KeyPressed;
  Read(kbd, ch);
end;


(* Inputs a single character. *)
function InputChar: Char;
var ch: Char;
begin
  Read(kbd, ch);
  if ch = chr(127) then
    Write(chr(8), ' ', chr(8)) (* backspace *)
  else
    Write(ch);    (* echo to screen *)
  InputChar := ch;
end; (* InputChar *)


(* Inputs a line until the Enter key is pressed. *)
(* Backspace can be used to correct the input.   *)
function InputLine(maxlen: Integer): WorkString;
var ch:  Char;
    cnt: Integer;
    s:   WorkString;
begin
  cnt := 0;
  s := '';
  repeat
    ch := InputChar;
    if ch = chr(127) then begin (* backspace *)
      Delete(s, cnt, 1);
      cnt := cnt - 1;
    end else
    if (ch > chr(31)) and (ch < chr(126)) then begin
      s := s + ch;
      cnt := cnt + 1;
    end;
  until (ch = chr(13)) or (cnt >= maxlen);
  
  InputLine := s;
end; (* InputLine *)


(* Inputs an 8.3 filename; input is finished with the Enter key. *)
(* Backspace can be used to correct the input.                   *)
function InputFileName: FileNameString;
var ch:         Char;
    cnt:        Integer;
    dotEntered: Boolean;
    len:        Integer;
    s:          WorkString;
begin
  cnt := 0;
  dotEntered := false;
  s := '';
  repeat
    ch := InputChar;
    if ch = chr(127) then begin (* backspace *)
      len := Length(s);
      if s(.len.) = '.' then begin
        dotEntered := false;
      end;
      Delete(s, len, 1);
      cnt := cnt - 1;
    end else
    if (ch >= 'A') and (ch <= 'Z') or
       (ch >= 'a') and (ch <= 'z') or
       (ch >= '0') and (ch <= '9') then begin
      if (cnt < 8) and not dotEntered or
         (cnt < 3) and dotEntered then begin
        s := s + ch;
        cnt := cnt + 1;
      end else begin
        Write(chr(8), ' ', chr(8));
      end;
    end else
    if ch = '.' then begin
      if not dotEntered then begin
        dotEntered := true;
        s := s + ch;
        cnt := 0;
      end;
    end;
  until ch = chr(13);
  
  InputFileName := s;
end; (* InputFileName *)

(* ------------------ Ende von INPUT.INC ----------------------- *)
